@@ -3,7 +3,10 @@ import pytest
 
 import solana.system_program as sp
 from solana.rpc.api import DataSliceOpt
+from solana.rpc.core import RPCException
+from solana.rpc.types import RPCError
 from solana.transaction import Transaction
+from spl.token.constants import WRAPPED_SOL_MINT
 
 from .utils import AIRDROP_AMOUNT, aconfirm_transaction, assert_valid_response, generate_expected_meta_after_airdrop
 
@@ -12,7 +15,7 @@ from .utils import AIRDROP_AMOUNT, aconfirm_transaction, assert_valid_response,
 @pytest.mark.asyncio
 async def test_request_air_drop(async_stubbed_sender, test_http_client_async):
     """Test air drop to async_stubbed_sender."""
-    resp = await test_http_client_async.request_airdrop(async_stubbed_sender.public_key(), AIRDROP_AMOUNT)
+    resp = await test_http_client_async.request_airdrop(async_stubbed_sender.public_key, AIRDROP_AMOUNT)
     assert_valid_response(resp)
     resp = await aconfirm_transaction(test_http_client_async, resp["result"])
     assert_valid_response(resp)
@@ -25,7 +28,7 @@ async def test_request_air_drop(async_stubbed_sender, test_http_client_async):
 async def test_request_air_drop_prefetched_blockhash(async_stubbed_sender_prefetched_blockhash, test_http_client_async):
     """Test air drop to async_stubbed_sender."""
     resp = await test_http_client_async.request_airdrop(
-        async_stubbed_sender_prefetched_blockhash.public_key(), AIRDROP_AMOUNT
+        async_stubbed_sender_prefetched_blockhash.public_key, AIRDROP_AMOUNT
     )
     assert_valid_response(resp)
     resp = await aconfirm_transaction(test_http_client_async, resp["result"])
@@ -41,7 +44,7 @@ async def test_request_air_drop_cached_blockhash(
 ):
     """Test air drop to async_stubbed_sender."""
     resp = await test_http_client_async_cached_blockhash.request_airdrop(
-        async_stubbed_sender_cached_blockhash.public_key(), AIRDROP_AMOUNT
+        async_stubbed_sender_cached_blockhash.public_key, AIRDROP_AMOUNT
     )
     assert_valid_response(resp)
     resp = await aconfirm_transaction(test_http_client_async_cached_blockhash, resp["result"])
@@ -50,6 +53,16 @@ async def test_request_air_drop_cached_blockhash(
     assert resp["result"]["meta"] == expected_meta
 
 
+@pytest.mark.integration
+@pytest.mark.asyncio
+async def test_send_invalid_transaction(test_http_client_async):
+    """Test sending an invalid transaction to localnet."""
+    # Create transfer tx to transfer lamports from stubbed sender to stubbed_receiver
+    with pytest.raises(RPCException) as exc_info:
+        await test_http_client_async.send_raw_transaction(b"foo")
+    assert exc_info.value.args[0].keys() == RPCError.__annotations__.keys()
+
+
 @pytest.mark.integration
 @pytest.mark.asyncio
 async def test_send_transaction_and_get_balance(async_stubbed_sender, async_stubbed_receiver, test_http_client_async):
@@ -58,7 +71,7 @@ async def test_send_transaction_and_get_balance(async_stubbed_sender, async_stub
     transfer_tx = Transaction().add(
         sp.transfer(
             sp.TransferParams(
-                from_pubkey=async_stubbed_sender.public_key(), to_pubkey=async_stubbed_receiver, lamports=1000
+                from_pubkey=async_stubbed_sender.public_key, to_pubkey=async_stubbed_receiver, lamports=1000
             )
         )
     )
@@ -92,7 +105,7 @@ async def test_send_transaction_and_get_balance(async_stubbed_sender, async_stub
     }
     assert resp["result"]["meta"] == expected_meta
     # Check balances
-    resp = await test_http_client_async.get_balance(async_stubbed_sender.public_key())
+    resp = await test_http_client_async.get_balance(async_stubbed_sender.public_key)
     assert_valid_response(resp)
     assert resp["result"]["value"] == 9999994000
     resp = await test_http_client_async.get_balance(async_stubbed_receiver)
@@ -110,7 +123,7 @@ async def test_send_transaction_prefetched_blockhash(
     transfer_tx = Transaction().add(
         sp.transfer(
             sp.TransferParams(
-                from_pubkey=async_stubbed_sender_prefetched_blockhash.public_key(),
+                from_pubkey=async_stubbed_sender_prefetched_blockhash.public_key,
                 to_pubkey=async_stubbed_receiver_prefetched_blockhash,
                 lamports=1000,
             )
@@ -146,7 +159,7 @@ async def test_send_transaction_prefetched_blockhash(
     }
     assert resp["result"]["meta"] == expected_meta
     # Check balances
-    resp = await test_http_client_async.get_balance(async_stubbed_sender_prefetched_blockhash.public_key())
+    resp = await test_http_client_async.get_balance(async_stubbed_sender_prefetched_blockhash.public_key)
     assert_valid_response(resp)
     assert resp["result"]["value"] == 9999994000
     resp = await test_http_client_async.get_balance(async_stubbed_receiver_prefetched_blockhash)
@@ -166,15 +179,20 @@ async def test_send_transaction_cached_blockhash(
     transfer_tx = Transaction().add(
         sp.transfer(
             sp.TransferParams(
-                from_pubkey=async_stubbed_sender_cached_blockhash.public_key(),
+                from_pubkey=async_stubbed_sender_cached_blockhash.public_key,
                 to_pubkey=async_stubbed_receiver_cached_blockhash,
                 lamports=1000,
             )
         )
     )
+    assert len(test_http_client_async_cached_blockhash.blockhash_cache.unused_blockhashes) == 0
+    assert len(test_http_client_async_cached_blockhash.blockhash_cache.used_blockhashes) == 0
     resp = await test_http_client_async_cached_blockhash.send_transaction(
         transfer_tx, async_stubbed_sender_cached_blockhash
     )
+    # we could have got a new blockhash or not depending on network latency and luck
+    assert len(test_http_client_async_cached_blockhash.blockhash_cache.unused_blockhashes) in (0, 1)
+    assert len(test_http_client_async_cached_blockhash.blockhash_cache.used_blockhashes) == 1
     assert_valid_response(resp)
     # Confirm transaction
     resp = await aconfirm_transaction(test_http_client_async_cached_blockhash, resp["result"])
@@ -204,16 +222,15 @@ async def test_send_transaction_cached_blockhash(
     }
     assert resp["result"]["meta"] == expected_meta
     # Check balances
-    resp = await test_http_client_async_cached_blockhash.get_balance(async_stubbed_sender_cached_blockhash.public_key())
+    resp = await test_http_client_async_cached_blockhash.get_balance(async_stubbed_sender_cached_blockhash.public_key)
     assert_valid_response(resp)
     assert resp["result"]["value"] == 9999994000
-    assert len(test_http_client_async_cached_blockhash.blockhash_cache.unused_blockhashes) == 1
 
     # Second transaction
     transfer_tx = Transaction().add(
         sp.transfer(
             sp.TransferParams(
-                from_pubkey=async_stubbed_sender_cached_blockhash.public_key(),
+                from_pubkey=async_stubbed_sender_cached_blockhash.public_key,
                 to_pubkey=async_stubbed_receiver_cached_blockhash,
                 lamports=2000,
             )
@@ -225,6 +242,9 @@ async def test_send_transaction_cached_blockhash(
     resp = await test_http_client_async_cached_blockhash.send_transaction(
         transfer_tx, async_stubbed_sender_cached_blockhash
     )
+    # we could have got a new blockhash or not depending on network latency and luck
+    assert len(test_http_client_async_cached_blockhash.blockhash_cache.unused_blockhashes) in (0, 1)
+    assert len(test_http_client_async_cached_blockhash.blockhash_cache.used_blockhashes) in (1, 2)
     assert_valid_response(resp)
     # Confirm transaction
     resp = await aconfirm_transaction(test_http_client_async_cached_blockhash, resp["result"])
@@ -246,11 +266,9 @@ async def test_send_transaction_cached_blockhash(
     }
     assert resp["result"]["meta"] == expected_meta
     # Check balances
-    resp = await test_http_client_async_cached_blockhash.get_balance(async_stubbed_sender_cached_blockhash.public_key())
+    resp = await test_http_client_async_cached_blockhash.get_balance(async_stubbed_sender_cached_blockhash.public_key)
     assert_valid_response(resp)
     assert resp["result"]["value"] == 9999987000
-    assert len(test_http_client_async_cached_blockhash.blockhash_cache.unused_blockhashes) == 1
-    assert len(test_http_client_async_cached_blockhash.blockhash_cache.used_blockhashes) == 1
 
 
 @pytest.mark.integration
@@ -267,7 +285,7 @@ async def test_send_raw_transaction_and_get_balance(
     transfer_tx = Transaction(recent_blockhash=recent_blockhash).add(
         sp.transfer(
             sp.TransferParams(
-                from_pubkey=async_stubbed_sender.public_key(), to_pubkey=async_stubbed_receiver, lamports=1000
+                from_pubkey=async_stubbed_sender.public_key, to_pubkey=async_stubbed_receiver, lamports=1000
             )
         )
     )
@@ -296,7 +314,7 @@ async def test_send_raw_transaction_and_get_balance(
     }
     assert resp["result"]["meta"] == expected_meta
     # Check balances
-    resp = await test_http_client_async.get_balance(async_stubbed_sender.public_key())
+    resp = await test_http_client_async.get_balance(async_stubbed_sender.public_key)
     assert_valid_response(resp)
     assert resp["result"]["value"] == 9999988000
     resp = await test_http_client_async.get_balance(async_stubbed_receiver)
@@ -495,6 +513,22 @@ async def test_get_supply(test_http_client_async):
     assert_valid_response(resp)
 
 
+@pytest.mark.integration
+@pytest.mark.asyncio
+async def test_get_token_largest_accounts(test_http_client_async):
+    """Test get token largest accounts."""
+    resp = await test_http_client_async.get_token_largest_accounts(WRAPPED_SOL_MINT)
+    assert_valid_response(resp)
+
+
+@pytest.mark.integration
+@pytest.mark.asyncio
+async def test_get_token_supply(test_http_client_async):
+    """Test get token supply."""
+    resp = await test_http_client_async.get_token_supply(WRAPPED_SOL_MINT)
+    assert_valid_response(resp)
+
+
 @pytest.mark.integration
 @pytest.mark.asyncio
 async def test_get_transaction_count(test_http_client_async):
@@ -515,13 +549,24 @@ async def test_get_version(test_http_client_async):
 @pytest.mark.asyncio
 async def test_get_account_info(async_stubbed_sender, test_http_client_async):
     """Test get_account_info."""
-    resp = await test_http_client_async.get_account_info(async_stubbed_sender.public_key())
+    resp = await test_http_client_async.get_account_info(async_stubbed_sender.public_key)
     assert_valid_response(resp)
-    resp = await test_http_client_async.get_account_info(async_stubbed_sender.public_key(), encoding="jsonParsed")
+    resp = await test_http_client_async.get_account_info(async_stubbed_sender.public_key, encoding="jsonParsed")
     assert_valid_response(resp)
-    resp = await test_http_client_async.get_account_info(
-        async_stubbed_sender.public_key(), data_slice=DataSliceOpt(1, 1)
-    )
+    resp = await test_http_client_async.get_account_info(async_stubbed_sender.public_key, data_slice=DataSliceOpt(1, 1))
+    assert_valid_response(resp)
+
+
+@pytest.mark.integration
+@pytest.mark.asyncio
+async def test_get_multiple_accounts(async_stubbed_sender, test_http_client_async):
+    """Test get_multiple_accounts."""
+    pubkeys = [async_stubbed_sender.public_key] * 2
+    resp = await test_http_client_async.get_multiple_accounts(pubkeys)
+    assert_valid_response(resp)
+    resp = await test_http_client_async.get_multiple_accounts(pubkeys, encoding="jsonParsed")
+    assert_valid_response(resp)
+    resp = await test_http_client_async.get_multiple_accounts(pubkeys, data_slice=DataSliceOpt(1, 1))
     assert_valid_response(resp)
 
 
