@@ -3,7 +3,10 @@ import pytest
 
 import solana.system_program as sp
 from solana.rpc.api import DataSliceOpt
+from solana.rpc.core import RPCException
+from solana.rpc.types import RPCError
 from solana.transaction import Transaction
+from spl.token.constants import WRAPPED_SOL_MINT
 
 from .utils import AIRDROP_AMOUNT, assert_valid_response, confirm_transaction, generate_expected_meta_after_airdrop
 
@@ -11,7 +14,7 @@ from .utils import AIRDROP_AMOUNT, assert_valid_response, confirm_transaction, g
 @pytest.mark.integration
 def test_request_air_drop(stubbed_sender, test_http_client):
     """Test air drop to stubbed_sender."""
-    resp = test_http_client.request_airdrop(stubbed_sender.public_key(), AIRDROP_AMOUNT)
+    resp = test_http_client.request_airdrop(stubbed_sender.public_key, AIRDROP_AMOUNT)
     assert_valid_response(resp)
     resp = confirm_transaction(test_http_client, resp["result"])
     assert_valid_response(resp)
@@ -22,7 +25,7 @@ def test_request_air_drop(stubbed_sender, test_http_client):
 @pytest.mark.integration
 def test_request_air_drop_prefetched_blockhash(stubbed_sender_prefetched_blockhash, test_http_client):
     """Test air drop to stubbed_sender."""
-    resp = test_http_client.request_airdrop(stubbed_sender_prefetched_blockhash.public_key(), AIRDROP_AMOUNT)
+    resp = test_http_client.request_airdrop(stubbed_sender_prefetched_blockhash.public_key, AIRDROP_AMOUNT)
     assert_valid_response(resp)
     resp = confirm_transaction(test_http_client, resp["result"])
     assert_valid_response(resp)
@@ -33,7 +36,7 @@ def test_request_air_drop_prefetched_blockhash(stubbed_sender_prefetched_blockha
 @pytest.mark.integration
 def test_request_air_drop_cached_blockhash(stubbed_sender_cached_blockhash, test_http_client):
     """Test air drop to stubbed_sender."""
-    resp = test_http_client.request_airdrop(stubbed_sender_cached_blockhash.public_key(), AIRDROP_AMOUNT)
+    resp = test_http_client.request_airdrop(stubbed_sender_cached_blockhash.public_key, AIRDROP_AMOUNT)
     assert_valid_response(resp)
     resp = confirm_transaction(test_http_client, resp["result"])
     assert_valid_response(resp)
@@ -41,14 +44,21 @@ def test_request_air_drop_cached_blockhash(stubbed_sender_cached_blockhash, test
     assert resp["result"]["meta"] == expected_meta
 
 
+@pytest.mark.integration
+def test_send_invalid_transaction(test_http_client):
+    """Test sending an invalid transaction to localnet."""
+    # Create transfer tx to transfer lamports from stubbed sender to stubbed_receiver
+    with pytest.raises(RPCException) as exc_info:
+        test_http_client.send_raw_transaction(b"foo")
+    assert exc_info.value.args[0].keys() == RPCError.__annotations__.keys()
+
+
 @pytest.mark.integration
 def test_send_transaction_and_get_balance(stubbed_sender, stubbed_receiver, test_http_client):
     """Test sending a transaction to localnet."""
     # Create transfer tx to transfer lamports from stubbed sender to stubbed_receiver
     transfer_tx = Transaction().add(
-        sp.transfer(
-            sp.TransferParams(from_pubkey=stubbed_sender.public_key(), to_pubkey=stubbed_receiver, lamports=1000)
-        )
+        sp.transfer(sp.TransferParams(from_pubkey=stubbed_sender.public_key, to_pubkey=stubbed_receiver, lamports=1000))
     )
     resp = test_http_client.send_transaction(transfer_tx, stubbed_sender)
     assert_valid_response(resp)
@@ -80,7 +90,7 @@ def test_send_transaction_and_get_balance(stubbed_sender, stubbed_receiver, test
     }
     assert resp["result"]["meta"] == expected_meta
     # Check balances
-    resp = test_http_client.get_balance(stubbed_sender.public_key())
+    resp = test_http_client.get_balance(stubbed_sender.public_key)
     assert_valid_response(resp)
     assert resp["result"]["value"] == 9999994000
     resp = test_http_client.get_balance(stubbed_receiver)
@@ -97,7 +107,7 @@ def test_send_transaction_prefetched_blockhash(
     transfer_tx = Transaction().add(
         sp.transfer(
             sp.TransferParams(
-                from_pubkey=stubbed_sender_prefetched_blockhash.public_key(),
+                from_pubkey=stubbed_sender_prefetched_blockhash.public_key,
                 to_pubkey=stubbed_receiver_prefetched_blockhash,
                 lamports=1000,
             )
@@ -136,7 +146,7 @@ def test_send_transaction_prefetched_blockhash(
     }
     assert resp["result"]["meta"] == expected_meta
     # Check balances
-    resp = test_http_client.get_balance(stubbed_sender_prefetched_blockhash.public_key())
+    resp = test_http_client.get_balance(stubbed_sender_prefetched_blockhash.public_key)
     assert_valid_response(resp)
     assert resp["result"]["value"] == 9999994000
     resp = test_http_client.get_balance(stubbed_receiver_prefetched_blockhash)
@@ -153,13 +163,18 @@ def test_send_transaction_cached_blockhash(
     transfer_tx = Transaction().add(
         sp.transfer(
             sp.TransferParams(
-                from_pubkey=stubbed_sender_cached_blockhash.public_key(),
+                from_pubkey=stubbed_sender_cached_blockhash.public_key,
                 to_pubkey=stubbed_receiver_cached_blockhash,
                 lamports=1000,
             )
         )
     )
+    assert len(test_http_client_cached_blockhash.blockhash_cache.unused_blockhashes) == 0
+    assert len(test_http_client_cached_blockhash.blockhash_cache.used_blockhashes) == 0
     resp = test_http_client_cached_blockhash.send_transaction(transfer_tx, stubbed_sender_cached_blockhash)
+    # we could have got a new blockhash or not depending on network latency and luck
+    assert len(test_http_client_cached_blockhash.blockhash_cache.unused_blockhashes) in (0, 1)
+    assert len(test_http_client_cached_blockhash.blockhash_cache.used_blockhashes) == 1
     assert_valid_response(resp)
     # Confirm transaction
     resp = confirm_transaction(test_http_client_cached_blockhash, resp["result"])
@@ -189,16 +204,15 @@ def test_send_transaction_cached_blockhash(
     }
     assert resp["result"]["meta"] == expected_meta
     # Check balances
-    resp = test_http_client_cached_blockhash.get_balance(stubbed_sender_cached_blockhash.public_key())
+    resp = test_http_client_cached_blockhash.get_balance(stubbed_sender_cached_blockhash.public_key)
     assert_valid_response(resp)
     assert resp["result"]["value"] == 9999994000
-    assert len(test_http_client_cached_blockhash.blockhash_cache.unused_blockhashes) == 1
 
     # Second transaction
     transfer_tx = Transaction().add(
         sp.transfer(
             sp.TransferParams(
-                from_pubkey=stubbed_sender_cached_blockhash.public_key(),
+                from_pubkey=stubbed_sender_cached_blockhash.public_key,
                 to_pubkey=stubbed_receiver_cached_blockhash,
                 lamports=2000,
             )
@@ -208,6 +222,9 @@ def test_send_transaction_cached_blockhash(
     assert_valid_response(resp)
     assert resp["result"]["value"] == 954
     resp = test_http_client_cached_blockhash.send_transaction(transfer_tx, stubbed_sender_cached_blockhash)
+    # we could have got a new blockhash or not depending on network latency and luck
+    assert len(test_http_client_cached_blockhash.blockhash_cache.unused_blockhashes) in (0, 1)
+    assert len(test_http_client_cached_blockhash.blockhash_cache.used_blockhashes) in (1, 2)
     assert_valid_response(resp)
     # Confirm transaction
     resp = confirm_transaction(test_http_client_cached_blockhash, resp["result"])
@@ -229,7 +246,7 @@ def test_send_transaction_cached_blockhash(
     }
     assert resp["result"]["meta"] == expected_meta
     # Check balances
-    resp = test_http_client_cached_blockhash.get_balance(stubbed_sender_cached_blockhash.public_key())
+    resp = test_http_client_cached_blockhash.get_balance(stubbed_sender_cached_blockhash.public_key)
     assert_valid_response(resp)
     assert resp["result"]["value"] == 9999987000
     assert len(test_http_client_cached_blockhash.blockhash_cache.unused_blockhashes) == 1
@@ -245,9 +262,7 @@ def test_send_raw_transaction_and_get_balance(stubbed_sender, stubbed_receiver,
     recent_blockhash = resp["result"]["value"]["blockhash"]
     # Create transfer tx transfer lamports from stubbed sender to stubbed_receiver
     transfer_tx = Transaction(recent_blockhash=recent_blockhash).add(
-        sp.transfer(
-            sp.TransferParams(from_pubkey=stubbed_sender.public_key(), to_pubkey=stubbed_receiver, lamports=1000)
-        )
+        sp.transfer(sp.TransferParams(from_pubkey=stubbed_sender.public_key, to_pubkey=stubbed_receiver, lamports=1000))
     )
     # Sign transaction
     transfer_tx.sign(stubbed_sender)
@@ -274,7 +289,7 @@ def test_send_raw_transaction_and_get_balance(stubbed_sender, stubbed_receiver,
     }
     assert resp["result"]["meta"] == expected_meta
     # Check balances
-    resp = test_http_client.get_balance(stubbed_sender.public_key())
+    resp = test_http_client.get_balance(stubbed_sender.public_key)
     assert_valid_response(resp)
     assert resp["result"]["value"] == 9999988000
     resp = test_http_client.get_balance(stubbed_receiver)
@@ -463,11 +478,37 @@ def test_get_version(test_http_client):
 @pytest.mark.integration
 def test_get_account_info(stubbed_sender, test_http_client):
     """Test get_account_info."""
-    resp = test_http_client.get_account_info(stubbed_sender.public_key())
+    resp = test_http_client.get_account_info(stubbed_sender.public_key)
     assert_valid_response(resp)
-    resp = test_http_client.get_account_info(stubbed_sender.public_key(), encoding="jsonParsed")
+    resp = test_http_client.get_account_info(stubbed_sender.public_key, encoding="jsonParsed")
     assert_valid_response(resp)
-    resp = test_http_client.get_account_info(stubbed_sender.public_key(), data_slice=DataSliceOpt(1, 1))
+    resp = test_http_client.get_account_info(stubbed_sender.public_key, data_slice=DataSliceOpt(1, 1))
+    assert_valid_response(resp)
+
+
+@pytest.mark.integration
+def test_get_multiple_accounts(stubbed_sender, test_http_client):
+    """Test get_multiple_accounts."""
+    pubkeys = [stubbed_sender.public_key] * 2
+    resp = test_http_client.get_multiple_accounts(pubkeys)
+    assert_valid_response(resp)
+    resp = test_http_client.get_multiple_accounts(pubkeys, encoding="jsonParsed")
+    assert_valid_response(resp)
+    resp = test_http_client.get_multiple_accounts(pubkeys, data_slice=DataSliceOpt(1, 1))
+    assert_valid_response(resp)
+
+
+@pytest.mark.integration
+def test_get_token_largest_accounts(test_http_client):
+    """Test get token largest accounts."""
+    resp = test_http_client.get_token_largest_accounts(WRAPPED_SOL_MINT)
+    assert_valid_response(resp)
+
+
+@pytest.mark.integration
+def test_get_token_supply(test_http_client):
+    """Test get token supply."""
+    resp = test_http_client.get_token_supply(WRAPPED_SOL_MINT)
     assert_valid_response(resp)
 
 
